==> ably_flutter-1.2.0.tar.gz: test_integration/lib/test/realtime/realtime_presence_subscribe.dart <==
import 'dart:async';

import 'package:ably_flutter/ably_flutter.dart';
import 'package:ably_flutter_example/provisioning.dart';

import '../../config/test_config.dart';
import '../../factory/reporter.dart';
import '../../utils/data.dart';
import '../../utils/encoders.dart';

final logMessages = <List<String>>[];

List<Map<String, dynamic>> _encode(List<PresenceMessage> messages) =>
    encodeList<PresenceMessage>(messages, encodePresenceMessage);

Future<Map<String, dynamic>> testRealtimePresenceSubscribe({
  Reporter reporter,
  Map<String, dynamic> payload,
}) async {
  reporter.reportLog('init start');
  final appKey = (await provision('sandbox-')).toString();
  final presence = Realtime(
    options: ClientOptions.fromKey(appKey)
      ..environment = 'sandbox'
      ..clientId = 'someClientId'
      ..logLevel = LogLevel.verbose
      ..logHandler =
          ({msg, exception}) => logMessages.add([msg, exception.toString()]),
  ).channels.get('test').presence;

  final allMessages = <PresenceMessage>[];
  final allMessagesSubscription = presence.subscribe().listen(allMessages.add);

  final enterMessages = <PresenceMessage>[];
  final enterMessagesSubscription = presence
      .subscribe(action: PresenceAction.enter)
      .listen(enterMessages.add);

  final enterUpdateMessages = <PresenceMessage>[];
  final enterUpdateMessagesSubscription = presence.subscribe(actions: [
    PresenceAction.enter,
    PresenceAction.update
  ]).listen(enterUpdateMessages.add);

  final partialMessages = <PresenceMessage>[];
  final partialMessagesSubscription =
      presence.subscribe().listen(partialMessages.add);

  // enter-update-leave sequence with different data types to check if
  // data received by listeners is intact
  await presence.enter(messagesToPublish.first[1]);
  for (var i = 1; i < messagesToPublish.length - 1; i++) {
    await presence.update(messagesToPublish[i][1]);
  }

  // Wait for the update event as it is asynchronously triggered.
  // Then cancelling partial subscription expecting it to not receive
  // further presence events.
  await Future.delayed(TestConstants.publishToHistoryDelay);
  await partialMessagesSubscription.cancel();

  await presence.leave(messagesToPublish.last[1]);

  // Wait for the leave event to be received by listeners.
  // Assuming, they'd turn out in 2 seconds.
  await Future.delayed(TestConstants.publishToHistoryDelay);
  await allMessagesSubscription.cancel();
  await enterMessagesSubscription.cancel();
  await enterUpdateMessagesSubscription.cancel();

  return {
    'allMessages': _encode(allMessages),
    'enterMessages': _encode(enterMessages),
    'enterUpdateMessages': _encode(enterUpdateMessages),
    'partialMessages': _encode(partialMessages),
    'log': logMessages,
  };
}

==> ably_flutter-1.2.1.tar.gz: test_integration/lib/test/realtime/realtime_presence_subscribe.dart <==
import 'dart:async';

import 'package:ably_flutter/ably_flutter.dart';
import 'package:ably_flutter_example/provisioning.dart';

import '../../config/test_constants.dart';
import '../../factory/reporter.dart';
import '../../utils/data.dart';
import '../../utils/encoders.dart';

final logMessages = <List<String?>>[];

List<Map<String, dynamic>> _encode(List<PresenceMessage> messages) =>
    encodeList<PresenceMessage>(messages, encodePresenceMessage);

Future<Map<String, dynamic>> testRealtimePresenceSubscribe({
  required Reporter reporter,
  Map<String, dynamic>? payload,
}) async {
  reporter.reportLog('init start');
  final appKey = (await provision('sandbox-')).toString();
  final presence = Realtime(
    options: ClientOptions.fromKey(appKey)
      ..environment = 'sandbox'
      ..clientId = 'someClientId'
      ..logLevel = LogLevel.verbose
      ..logHandler =
          ({msg, exception}) => logMessages.add([msg, exception.toString()]),
  ).channels.get('test').presence;

  final allMessages = <PresenceMessage>[];
  final allMessagesSubscription = presence.subscribe().listen(allMessages.add);

  final enterMessages = <PresenceMessage>[];
  final enterMessagesSubscription = presence
      .subscribe(action: PresenceAction.enter)
      .listen(enterMessages.add);

  final enterUpdateMessages = <PresenceMessage>[];
  final enterUpdateMessagesSubscription = presence.subscribe(actions: [
    PresenceAction.enter,
    PresenceAction.update
  ]).listen(enterUpdateMessages.add);

  final partialMessages = <PresenceMessage>[];
  final partialMessagesSubscription =
      presence.subscribe().listen(partialMessages.add);

  // enter-update-leave sequence with different data types to check if
  // data received by listeners is intact
  await presence.enter(messagesToPublish.first[1]);
  for (var i = 1; i < messagesToPublish.length - 1; i++) {
    await presence.update(messagesToPublish[i][1]);
  }

  // Wait for the update event as it is asynchronously triggered.
  // Then cancelling partial subscription expecting it to not receive
  // further presence events.
  await Future.delayed(TestConstants.publishToHistoryDelay);
  await partialMessagesSubscription.cancel();

  await presence.leave(messagesToPublish.last[1]);

  // Wait for the leave event to be received by listeners.
  // Assuming, they'd turn out in 2 seconds.
  await Future.delayed(TestConstants.publishToHistoryDelay);
  await allMessagesSubscription.cancel();
  await enterMessagesSubscription.cancel();
  await enterUpdateMessagesSubscription.cancel();

  return {
    'allMessages': _encode(allMessages),
    'enterMessages': _encode(enterMessages),
    'enterUpdateMessages': _encode(enterUpdateMessages),
    'partialMessages': _encode(partialMessages),
    'log': logMessages,
  };
}
